<?php

require_once('db.php');
require_once('../model/Response.php');
require_once('../function/checkJson.php');
require_once('../function/checkValue.php');
require_once('../function/checkTokenExpiry.php');

try {
    $writeDB = DB::connectWriteDB();
    $readDB = DB::connectReadDB();
} catch (PDOException $ex) {
    error_log("Database Connection error " . $ex, 0);
    $response = new Response();
    $response->setHttpStatusCode(500);
    $response->setSuccess(false);
    $response->addMessage("Database Connection error" . $ex->getMessage());
    $response->send();
    exit;
}


if (array_key_exists("serviceId", $_GET)) {
    $pageLimit = 10;
    $page = $_GET['page'];
    $serviceId = $_GET['serviceId'];

    checkValue($page, 'Page number provided invalid.');
    checkValue($jsonData->serviceId, 'Service Id provided invalid.');

    $pageIndex = ($page - 1) * $pageLimit;

    if ($_SERVER['REQUEST_METHOD'] == 'GET') {
        $query = $readDB->prepare('SELECT count(re.*) as count_all
                                FROM tbl_service_review as re
                                left join tbl_service_rating as ra on re.id = ra.review_id
                                where re.service_id = :serviceId');
        $query->bindParam(':serviceId', $serviceId, PDO::PARAM_INT);
        $query->execute();

        $rowCount = $query->rowCount();

        if ($rowCount == 0) {
            $response = new Response();
            $response->setHttpStatusCode(404);
            $response->setSuccess(false);
            $response->addMessage("No Data found");
            $response->send();
            exit;
        }

        $row = $query->fetch(PDO::FETCH_ASSOC);

        $totalRecords = intval($row['count_all']);

        $totalPages = ceil($totalRecords / $pageLimit);

        try {
            $query = $readDB->prepare('SELECT re.id, re.service_id, re.comment, ra.rate, re.created_date
                                    FROM tbl_service_review as re
                                    left join tbl_service_rating as ra on re.id = ra.review_id
                                    where re.service_id = :serviceId 
                                    limit :pageIndex, :pageLimit');
            $query->bindParam(':serviceId', $serviceId, PDO::PARAM_INT);
            $query->bindParam(':pageIndex', $pageIndex, PDO::PARAM_INT);
            $query->bindParam(':pageLimit', $pageLimit, PDO::PARAM_INT);
            $query->execute();

            $rowCount = $query->rowCount();

            if ($rowCount == 0) {
                $response = new Response();
                $response->setHttpStatusCode(404);
                $response->setSuccess(false);
                $response->addMessage("No Comments Data found");
                $response->send();
                exit;
            }

            $dataArray = array();

            while ($row = $query->fetch(PDO::FETCH_ASSOC)) {
                $data = array();
                $data['id'] = $row['id'];
                $data['serviceId'] = $row['service_id'];
                $data['comment'] = $row['comment'];
                $data['rate'] = $row['rate'];
                $data['createdDate'] = $row['created_date'];
                
                $dataArray[] = $data;
            }

            $response = new Response();
            $response->setHttpStatusCode(200);
            $response->setSuccess(true);
            $response->toCache(true);
            $response->setData($dataArray);
            $response->send();
        } catch (PDOException $ex) {
            error_log("Database query error " . $ex, 0);
            $response = new Response();
            $response->setHttpStatusCode(500);
            $response->setSuccess(false);
            $response->addMessage("Failed to get Comments.");
            $response->send();
            exit;
        }
    } else {
        $response = new Response();
        $response->setHttpStatusCode(405);
        $response->setSuccess(false);
        $response->addMessage("Method not allowed");
        $response->send();
        exit;
    }
} elseif (empty($_GET)) {
    checkTokenExpiry();

    if ($_SERVER['REQUEST_METHOD'] == 'POST') {
        $rowPostData = file_get_contents('php://input');

        $jsonData = checkJson($rowPostData);

        checkValue($jsonData->serviceId, 'Service Id provided invalid.');
        checkValue($jsonData->userId, 'User Id provided invalid.');
        checkValue($jsonData->comment, 'Comment provided invalid.');
        checkValue($jsonData->rate, 'Rate provided invalid.');

        $serviceId = $jsonData->serviceId;
        $userId = $jsonData->userId;
        $comment = $jsonData->comment;
        $rate = $jsonData->rate;

        try {
            $query = $readDB->prepare('SELECT id 
                                        from tbl_service 
                                        where id = :serviceId');
            $query->bindParam(':serviceId', $serviceId, PDO::PARAM_INT);
            $query->execute();

            $rowCount = $query->rowCount();

            if ($rowCount === 0) {
                $response = new Response();
                $response->setHttpStatusCode(404);
                $response->setSuccess(false);
                $response->addMessage("Service does not exist.");
                $response->send();
                exit;
            }

            
            $query = $writeDB->prepare('INSERT into tbl_service_review (service_id, comment, created_by)
                                        values (:serviceId, :comment, :userId)');
            $query->bindParam(':serviceId', $serviceId, PDO::PARAM_INT);
            $query->bindParam(':comment', $comment, PDO::PARAM_STR);
            $query->bindParam(':userId', $userId, PDO::PARAM_STR);
            $query->execute();

            $rowCount = $query->rowCount();

            if ($rowCount === 0) {
                $response = new Response();
                $response->setHttpStatusCode(500);
                $response->setSuccess(false);
                $response->addMessage("Error creating Comments.");
                $response->send();
                exit;
            }

            $tableId = $writeDB->lastInsertId();

            $query = $writeDB->prepare('INSERT into tbl_service_rating (review_id, service_id, rate, created_by)
                                        values (:tableId, :serviceId, :rate, :userId)');
            $query->bindParam(':tableId', $tableId, PDO::PARAM_INT);
            $query->bindParam(':serviceId', $serviceId, PDO::PARAM_INT);
            $query->bindParam(':rate', $comment, PDO::PARAM_INT);
            $query->bindParam(':userId', $userId, PDO::PARAM_STR);
            $query->execute();

            $rowCount = $query->rowCount();

            if ($rowCount === 0) {
                $response = new Response();
                $response->setHttpStatusCode(500);
                $response->setSuccess(false);
                $response->addMessage("Error creating rating.");
                $response->send();
                exit;
            }

            $returnData = array();
            $returnData['id'] = $tableId;
            $returnData['serviceId'] = $serviceId;
            $returnData['userId'] = $userId;
            $returnData['comment'] = $comment;
            $returnData['rate'] = $rate;

            $response = new Response();
            $response->setHttpStatusCode(201);
            $response->setSuccess(true);
            $response->addMessage("Comments created successfully");
            $response->setData($returnData);
            $response->send();
            exit;
        } catch (PDOException $ex) {
            error_log("Database query error " . $ex, 0);
            $response = new Response();
            $response->setHttpStatusCode(500);
            $response->setSuccess(false);
            $response->addMessage("Failed to create Comments.");
            $response->send();
            exit;
        }
    }
    //  elseif ($_SERVER['REQUEST_METHOD'] == 'PATCH') {
    //     $rowPostData = file_get_contents('php://input');

    //     $jsonData = checkJson($rowPostData);

    //     checkValue($jsonData->serviceId, 'Service Id provided invalid.');
    //     checkValue($jsonData->userId, 'User Id provided invalid.');
    //     checkValue($jsonData->comment, 'Comment provided invalid.');
    //     checkValue($jsonData->rate, 'Rate provided invalid.');
    //     checkValue($jsonData->id, 'Id provided invalid.');

    //     $serviceId = $jsonData->serviceId;
    //     $userId = $jsonData->userId;
    //     $comment = $jsonData->comment;
    //     $rate = $jsonData->rate;
    //     $id = $jsonData->id;

    //     try {
    //         $query = $readDB->prepare('SELECT id
    //                                     from tbl_service
    //                                     where id = :serviceId');
    //         $query->bindParam(':serviceId', $serviceId, PDO::PARAM_INT);
    //         $query->execute();

    //         $rowCount = $query->rowCount();

    //         if ($rowCount === 0) {
    //             $response = new Response();
    //             $response->setHttpStatusCode(404);
    //             $response->setSuccess(false);
    //             $response->addMessage("Service does not exist.");
    //             $response->send();
    //             exit;
    //         }

    //         $query = $readDB->prepare('SELECT id
    //                                     from tbl_service_review
    //                                     where id = :id');
    //         $query->bindParam(':id', $id, PDO::PARAM_INT);
    //         $query->execute();

    //         $rowCount = $query->rowCount();

    //         if ($rowCount === 0) {
    //             $response = new Response();
    //             $response->setHttpStatusCode(404);
    //             $response->setSuccess(false);
    //             $response->addMessage("Comments does not exist.");
    //             $response->send();
    //             exit;
    //         }

    //         $query = $writeDB->prepare('UPDATE tbl_service_review
    //                                     set comment = :comment,
    //                                     edited_by = :userId
    //                                     where id = :id');
    //         $query->bindParam(':comment', $comment, PDO::PARAM_STR);
    //         $query->bindParam(':userId', $userId, PDO::PARAM_STR);
    //         $query->bindParam(':id', $id, PDO::PARAM_INT);
    //         $query->execute();

    //         $rowCount = $query->rowCount();

    //         if ($rowCount == 0) {
    //             $response = new Response();
    //             $response->setHttpStatusCode(500);
    //             $response->setSuccess(false);
    //             $response->addMessage("Error updating data Comments.");
    //             $response->send();
    //             exit;
    //         }

    //         $query = $writeDB->prepare('UPDATE tbl_service_rating
    //                                     set rate = :rate,
    //                                     edited_by = :userId
    //                                     where id = :id');
    //         $query->bindParam(':rate', $rate, PDO::PARAM_INT);
    //         $query->bindParam(':userId', $userId, PDO::PARAM_STR);
    //         $query->bindParam(':id', $id, PDO::PARAM_INT);
    //         $query->execute();

    //         $rowCount = $query->rowCount();

    //         if ($rowCount == 0) {
    //             $response = new Response();
    //             $response->setHttpStatusCode(500);
    //             $response->setSuccess(false);
    //             $response->addMessage("Error updating data Comments.");
    //             $response->send();
    //             exit;
    //         }


    //         $returnData = array();
    //         $returnData['id'] = $id;
    //         $returnData['serviceId'] = $serviceId;
    //         $returnData['userId'] = $userId;
    //         $returnData['comment'] = $comment;
    //         $returnData['rate'] = $rate;

    //         $response = new Response();
    //         $response->setHttpStatusCode(200);
    //         $response->setSuccess(true);
    //         $response->addMessage("Comments updated successfully");
    //         $response->setData($returnData);
    //         $response->send();
    //         exit;
    //     } catch (PDOException $ex) {
    //         error_log("Database query error " . $ex, 0);
    //         $response = new Response();
    //         $response->setHttpStatusCode(500);
    //         $response->setSuccess(false);
    //         $response->addMessage("Failed to update Comments.");
    //         $response->send();
    //         exit;
    //     }
    // }
    else {
        $response = new Response();
        $response->setHttpStatusCode(405);
        $response->setSuccess(false);
        $response->addMessage("Method not allowed");
        $response->send();
        exit;
    }
} else {
    $response = new Response();
    $response->setHttpStatusCode(405);
    $response->setSuccess(false);
    $response->addMessage("Method not allowed");
    $response->send();
    exit;
}
