<?php

require_once('db.php');
require_once('../model/Response.php');
require_once('../function/checkJson.php');
require_once('../function/checkValue.php');
require_once('../function/checkTokenExpiry.php');

try {
    $writeDB = DB::connectWriteDB();
    $readDB = DB::connectReadDB();
} catch (PDOException $ex) {
    error_log("Database Connection error : " . $ex, 0);
    $response = new Response();
    $response->setHttpStatusCode(500);
    $response->setSuccess(false);
    $response->addMessage("Database Connection error");
    $response->send();
    exit;
}

if (empty($_GET)) {
    if ($_SERVER['REQUEST_METHOD'] == 'POST') {
        sleep(1);

        $rowPostData = file_get_contents('php://input');

        $jsonData = checkJson($rowPostData);

        checkValue($jsonData->username, 'Username provided invalid.');
        checkValue($jsonData->password, 'Password provided invalid.');

        $username = $jsonData->username;
        $password = $jsonData->password;
        $deviceId = isset($jsonData->deviceId) ? $jsonData->deviceId: '00000';

        try {
            $query = $readDB->prepare('SELECT user_id, password, username
                                        from tbl_user_login 
                                        where username = :username and disabled_date is null');
            $query->bindParam(':username', $username, PDO::PARAM_STR);
            $query->execute();

            $rowCount = $query->rowCount();

            if ($rowCount === 0) {
                $response = new Response();
                $response->setHttpStatusCode(401);
                $response->setSuccess(false);
                $response->addMessage("Username or Password is incorrect");
                $response->send();
                exit;
            }

            $row = $query->fetch(PDO::FETCH_ASSOC);

            $returnUserId = $row['user_id'];
            $returnPassword = $row['password'];
            $returnUserName = $row['username'];

            if (!password_verify($password, $returnPassword)) {
                $response = new Response();
                $response->setHttpStatusCode(401);
                $response->setSuccess(false);
                $response->addMessage("Username or Password is incorrect");
                $response->send();
                exit;
            }

            $query = $writeDB->prepare('DELETE 
                                        from tbl_session 
                                        where user_id = :userId');
            $query->bindParam(':userId', $returnUserId, PDO::PARAM_STR);
            $query->execute();

            $token = base64_encode(bin2hex(openssl_random_pseudo_bytes(24)).time());
            $token_expiry_seconds = 1200;

            $query = $writeDB->prepare("INSERT into tbl_session 
                                        (user_id, token, device_id, expiry_date) 
                                        values 
                                        (:userId, :token, :deviceId, NOW() + interval '" . $token_expiry_seconds .  "' second)");
            $query->bindParam(':userId', $returnUserId, PDO::PARAM_STR);
            $query->bindParam(':token', $token, PDO::PARAM_STR);
            $query->bindParam(':deviceId', $deviceId, PDO::PARAM_STR);
            $query->execute();

            $rowCount = $query->rowCount();

            if ($rowCount == 0) {
                $response = new Response();
                $response->setHttpStatusCode(500);
                $response->setSuccess(false);
                $response->addMessage("Error creating session.");
                $response->send();
                exit;
            }

            $lastSessionID = $writeDB->lastInsertId();
            $sessionId= intval($lastSessionID);

            $query = $readDB->prepare('SELECT user_id, first_name, last_name, phone_number, company_name, status_id, role
                                        from vw_user_detail
                                        where user_id = :userId and status_id = 1');
            $query->bindParam(':userId', $returnUserId, PDO::PARAM_STR);
            $query->execute();

            $rowCount = $query->rowCount();

            if ($rowCount === 0) {
                $response = new Response();
                $response->setHttpStatusCode(400);
                $response->setSuccess(false);
                $response->addMessage("User not active");
                $response->send();
                exit;
            }

            $row = $query->fetch(PDO::FETCH_ASSOC);

            // if ($row['status_id'] === 2) {
            //     $response = new Response();
            //     $response->setHttpStatusCode(401);
            //     $response->setSuccess(false);
            //     $response->addMessage("User not authorized");
            //     $response->send();
            //     exit;
            // }

            $returnData = array();
            $returnData['userId'] = $returnUserId;
            $returnData['username'] = $returnUserName;
            $returnData['firstName'] = $row['first_name'];
            $returnData['lastName'] = $row['last_name'];
            $returnData['phoneNumber'] = $row['phone_number'];
            $returnData['companyName'] = $row['company_name'];
            $returnData['roleId'] = $row['role'];
            $returnData['token'] = $token;
            $returnData['tokenExpiry'] = $token_expiry_seconds;
            $returnData['sessionId'] = $sessionId;

            $response = new Response();
            $response->setHttpStatusCode(201);
            $response->setSuccess(true);
            $response->setData($returnData);
            $response->send();
            exit;
        } catch (PDOException $ex) {
            error_log("log in error: " . $ex, 0);
            $response = new Response();
            $response->setHttpStatusCode(500);
            $response->setSuccess(false);
            $response->addMessage("There was an issue logging in." . $ex);
            $response->send();
            exit;
        }
    } elseif ($_SERVER['REQUEST_METHOD'] == 'PATCH') {
        checkTokenExpiry();

        $rowPostData = file_get_contents('php://input');

        $jsonData = checkJson($rowPostData);

        checkValue($jsonData->newPassword, 'New Password provided invalid.');
        checkValue($jsonData->oldPassword, 'Old Password provided invalid.');
        checkValue($jsonData->userId, 'User Id provided invalid.');

        $newPassword = $jsonData->newPassword;
        $oldPassword = $jsonData->oldPassword;
        $userId = $jsonData->userId;

        try {
            $query = $readDB->prepare('SELECT user_id, password, user_name
                                        from tbl_user_login 
                                        where user_id = :userId and deleted_date is null');
            $query->bindParam(':userId', $userId, PDO::PARAM_STR);
            $query->execute();

            $rowCount = $query->rowCount();

            if ($rowCount === 0) {
                $response = new Response();
                $response->setHttpStatusCode(401);
                $response->setSuccess(false);
                $response->addMessage("User does not exist.");
                $response->send();
                exit;
            }

            $row = $query->fetch(PDO::FETCH_ASSOC);

            $returned_username = $row['user_name'];
            $returned_password = $row['password'];

            if (!password_verify($oldPassword, $returned_password)) {
                $response = new Response();
                $response->setHttpStatusCode(400);
                $response->setSuccess(false);
                $response->addMessage("Old password provided is incorrect");
                $response->send();
                exit;
            }

            $hashed_password = password_hash($newPassword, PASSWORD_DEFAULT);

            $query = $writeDB->prepare('UPDATE tbl_user_login 
                                        set password = :password 
                                        where user_id = :userId ');
            $query->bindParam(':password', $hashed_password, PDO::PARAM_STR);
            $query->bindParam(':userId', $userId, PDO::PARAM_STR);
            $query->execute();

            $rowCount = $query->rowCount();

            if ($rowCount == 0) {
                $response = new Response();
                $response->setHttpStatusCode(500);
                $response->setSuccess(false);
                $response->addMessage("Error updating password.");
                $response->send();
                exit;
            }

            $tableId = $writeDB->lastInsertId();

            $response = new Response();
            $response->setHttpStatusCode(200);
            $response->setSuccess(true);
            $response->addMessage("Password updated successfully");
            $response->send();
            exit;
        } catch (PDOException $ex) {
            error_log("There was an issue updating password, please try again: " . $ex, 0);
            $response = new Response();
            $response->setHttpStatusCode(500);
            $response->setSuccess(false);
            $response->addMessage("There was an issue updating password, please try again");
            $response->send();
            exit;
        }
    } elseif ($_SERVER['REQUEST_METHOD'] == 'DELETE') {
        checkTokenExpiry();

        $rowPostData = file_get_contents('php://input');

        $jsonData = checkJson($rowPostData);

        checkValue($jsonData->userId, 'User Id provided invalid.');
        checkValue($jsonData->roleId, 'User Id provided invalid.');

        $userId = $jsonData->userId;
        $roleId = $jsonData->roleId;

        try {
            $query = $readDB->prepare('SELECT user_id, password, user_name
                                        from tbl_user_login 
                                        where user_id = :userId and deleted_date is null');
            $query->bindParam(':userId', $userId, PDO::PARAM_STR);
            $query->execute();

            $rowCount = $query->rowCount();

            if ($rowCount === 0) {
                $response = new Response();
                $response->setHttpStatusCode(401);
                $response->setSuccess(false);
                $response->addMessage("User does not exist.");
                $response->send();
                exit;
            }

            $query = $writeDB->prepare('UPDATE tbl_user_login 
                                        set disabled_date = CURDATE()
                                        where user_id = :userId ');
            $query->bindParam(':userId', $userId, PDO::PARAM_STR);
            $query->execute();

            $rowCount = $query->rowCount();

            if ($rowCount == 0) {
                $response = new Response();
                $response->setHttpStatusCode(500);
                $response->setSuccess(false);
                $response->addMessage("Error deleting account.");
                $response->send();
                exit;
            }

            $tableId = $writeDB->lastInsertId();

            $response = new Response();
            $response->setHttpStatusCode(200);
            $response->setSuccess(true);
            $response->addMessage("successful");
            $response->send();
            exit;
        } catch (PDOException $ex) {
            error_log("There was an issue updating password, please try again: " . $ex, 0);
            $response = new Response();
            $response->setHttpStatusCode(500);
            $response->setSuccess(false);
            $response->addMessage("There was an issue updating password, please try again");
            $response->send();
            exit;
        }
    } else {
        $response = new Response();
        $response->setHttpStatusCode(405);
        $response->setSuccess(false);
        $response->addMessage("Method not allowed");
        $response->send();
        exit;
    }
} else {
    $response = new Response();
    $response->setHttpStatusCode(405);
    $response->setSuccess(false);
    $response->addMessage("Method not allowed");
    $response->send();
    exit;
}
